package com.palmnest.application.core.util;

import org.apache.commons.lang.StringUtils;

import net.sourceforge.pinyin4j.PinyinHelper;
import net.sourceforge.pinyin4j.format.HanyuPinyinCaseType;
import net.sourceforge.pinyin4j.format.HanyuPinyinOutputFormat;
import net.sourceforge.pinyin4j.format.HanyuPinyinToneType;
import net.sourceforge.pinyin4j.format.HanyuPinyinVCharType;
import net.sourceforge.pinyin4j.format.exception.BadHanyuPinyinOutputFormatCombination;

public class FirstLetterUtil {
	/**
	 * 得到 全拼
	 *
	 * @param src
	 * @return
	 */
	public static String getPingYin(String src) {
		char[] t1 = null;
		t1 = src.toCharArray();
		String[] t2 = new String[t1.length];
		HanyuPinyinOutputFormat t3 = new HanyuPinyinOutputFormat();
		t3.setCaseType(HanyuPinyinCaseType.LOWERCASE);
		t3.setToneType(HanyuPinyinToneType.WITHOUT_TONE);
		t3.setVCharType(HanyuPinyinVCharType.WITH_V);
		String t4 = "";
		int t0 = t1.length;
		try {
			for (int i = 0; i < t0; i++) {
				// 判断是否为汉字字符
				if (java.lang.Character.toString(t1[i]).matches("[\\u4E00-\\u9FA5]+")) {
					t2 = PinyinHelper.toHanyuPinyinStringArray(t1[i], t3);
					t4 += t2[0];
				} else {
					t4 += java.lang.Character.toString(t1[i]);
				}
			}
			return t4;
		} catch (BadHanyuPinyinOutputFormatCombination e1) {
			e1.printStackTrace();
		}
		return t4;
	}

	/**
	 * 得到中文首字母
	 *
	 * @param str
	 * @return
	 */
	public static String getPinYinHeadChar(String str) {

		String convert = "";
		for (int j = 0; j < str.length(); j++) {
			char word = str.charAt(j);
			String[] pinyinArray = PinyinHelper.toHanyuPinyinStringArray(word);
			if (pinyinArray != null) {
				convert += pinyinArray[0].charAt(0);
			} else {
				convert += word;
			}
		}

		return convert;
	}

	/**
	 * 得到中文首字母+编码
	 *
	 * @param str                               城市中文（比如深圳）
	 * @param customerNumber（客户编号，该项目历史最新的客户编号）
	 * @return
	 */
	public static String getPinYinHeadCharCode(String str, String customerNumber) {
		String convert = "";
		for (int j = 0; j < str.length(); j++) {
			char word = str.charAt(j);
			String[] pinyinArray = PinyinHelper.toHanyuPinyinStringArray(word);
			if (pinyinArray != null) {
				convert += pinyinArray[0].charAt(0);
			} else {
				convert += word;
			}
		}

		String number = "";//存储数字
		if (StringUtils.isNotEmpty(customerNumber)) {
			number = customerNumber.replaceAll("[^0-9]", "");//获得数字
		}

		if (StringUtils.isNotEmpty(number)) {
			number = String.valueOf(Integer.valueOf(number) + 1);
		} else {
			number = "0001";
		}
		convert = convert + padRight(number, 4, 0);
		return convert;
	}

	/**
	 * 将字符串转移为ASCII码
	 *
	 * @param cnStr
	 * @return
	 */
	public static String getCnASCII(String cnStr) {
		StringBuffer strBuf = new StringBuffer();
		byte[] bGBK = cnStr.getBytes();
		for (int i = 0; i < bGBK.length; i++) {
			strBuf.append(Integer.toHexString(bGBK[i] & 0xff));
		}
		return strBuf.toString();
	}

	/**
	 * 左补位，右对齐
	 *
	 * @param oriStr 原字符串
	 * @param len    目标字符串长度
	 * @param alexin 补位字符
	 * @return 目标字符串
	 */
	public static String padLeft(String oriStr, int len, int alexin) {
		int strlen = oriStr.length();
		String str = "";
		if (strlen < len) {
			for (int i = 0; i < len - strlen; i++) {
				str = str + alexin;
			}
		}
		str = oriStr + str;
		return str;
	}

	/**
	 * 右补位，左对齐
	 *
	 * @param oriStr 原字符串
	 * @param len    目标字符串长度
	 * @param alexin 补位字符
	 * @return 目标字符串
	 */
	public static String padRight(String oriStr, int len, int alexin) {
		int strlen = oriStr.length();
		String str = "";
		if (strlen < len) {
			for (int i = 0; i < len - strlen; i++) {
				str = str + alexin;
			}
		}
		str = str + oriStr;
		return str;
	}

	public static void main(String[] args) {
		String dda = "10.11";
		dda = dda.substring(0, dda.indexOf("."));
		System.out.println(dda);
	}
}