package com.palmnest.application.core.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


import org.apache.commons.lang.StringUtils;

public class RegularPattern {
	/**
	 * 检验手机号码的正确性
	 *
	 * @param mobiles
	 * @return
	 */
	public static boolean isMobileNO(String mobiles) {
		Pattern p = Pattern.compile("^((17[0-9])|(14[0-9])|(13[0-9])|(15[0-9])|(18[0-9])|(16[0-9])|(19[0-9]))\\d{8}$");
		Matcher m = p.matcher(mobiles);
		System.out.println(m.matches() + "---");
		return m.matches();
	}


	/**
	 * 检验手机号码的正确性
	 *
	 * @param mobiles(以逗号隔开)
	 * @return
	 */
	public static Map<String, Object> isMobileNOList(String mobiles) {
		int bugCount = 0;
		StringBuffer bugStr = new StringBuffer("");
		Map<String, Object> map = new HashMap<String, Object>();
		Pattern p = Pattern.compile("^((17[0-9])|(14[0-9])|(13[0-9])|(15[0-9])|(18[0-9])|(16[0-9])|(19[0-9]))\\d{8}$");
		if (StringUtils.isNotEmpty(mobiles)) {
			String[] tel = mobiles.split(",");
			System.out.println(tel.length);
			for (int i = 0; i < tel.length; i++) {
				Matcher m = p.matcher(tel[i]);
				if (!m.matches()) {
					bugStr.append("第" + (i + 1) + "个电话号码检测到异常，电话号码为：" + tel[i] + ";");
					bugCount++;
				}
			}
		}
		map.put("bugCount", bugCount);
		map.put("bugStr", bugStr);
		return map;
	}

	/**
	 * 检测邮箱
	 *
	 * @param email
	 * @return
	 */
	public static boolean checkEmail(String email) {
		boolean flag = false;
		String check = "^([a-z0-9A-Z]+[-|_|\\.]?)+[a-z0-9A-Z]@([a-z0-9A-Z]+(-[a-z0-9A-Z]+)?\\.)+[a-zA-Z]{2,}$";
		Pattern regex = Pattern.compile(check);
		Matcher matcher = regex.matcher(email);
		flag = matcher.matches();
		return flag;
	}

	public static Map<String, Object> checkEmailList(String emails) {
		int bugCount = 0;
		StringBuffer bugStr = new StringBuffer("");
		Map<String, Object> map = new HashMap<String, Object>();
		String check = "^([a-z0-9A-Z]+[-|_|\\.]?)+[a-z0-9A-Z]@([a-z0-9A-Z]+(-[a-z0-9A-Z]+)?\\.)+[a-zA-Z]{2,}$";
		Pattern regex = Pattern.compile(check);
		if (StringUtils.isNotEmpty(emails)) {
			String[] tel = emails.split(",");
			for (int i = 0; i < tel.length; i++) {
				Matcher m = regex.matcher(tel[i]);
				if (!m.matches()) {
					bugStr.append("第" + (i + 1) + "个邮箱检测到异常，邮箱为：" + tel[i] + ";");
					bugCount++;
				}
			}
		}
		map.put("bugCount", bugCount);
		map.put("bugStr", bugStr);
		return map;
	}

	/**
	 * 检测身份证
	 *
	 * @param IDNumber
	 * @return
	 */
	public static boolean isIDNumber(String IDNumber) {
		if (IDNumber == null || "".equals(IDNumber)) {
			return false;
		}
		// 定义判别用户身份证号的正则表达式（15位或者18位，最后一位可以为字母）
		String regularExpression = "(^[1-9]\\d{5}(18|19|20)\\d{2}((0[1-9])|(10|11|12))(([0-2][1-9])|10|20|30|31)\\d{3}[0-9Xx]$)|" +
				"(^[1-9]\\d{5}\\d{2}((0[1-9])|(10|11|12))(([0-2][1-9])|10|20|30|31)\\d{3}$)";
		//假设18位身份证号码:41000119910101123X  410001 19910101 123X
		//^开头
		//[1-9] 第一位1-9中的一个      4
		//\\d{5} 五位数字           10001（前六位省市县地区）
		//(18|19|20)                19（现阶段可能取值范围18xx-20xx年）
		//\\d{2}                    91（年份）
		//((0[1-9])|(10|11|12))     01（月份）
		//(([0-2][1-9])|10|20|30|31)01（日期）
		//\\d{3} 三位数字            123（第十七位奇数代表男，偶数代表女）
		//[0-9Xx] 0123456789Xx其中的一个 X（第十八位为校验值）
		//$结尾

		//假设15位身份证号码:410001910101123  410001 910101 123
		//^开头
		//[1-9] 第一位1-9中的一个      4
		//\\d{5} 五位数字           10001（前六位省市县地区）
		//\\d{2}                    91（年份）
		//((0[1-9])|(10|11|12))     01（月份）
		//(([0-2][1-9])|10|20|30|31)01（日期）
		//\\d{3} 三位数字            123（第十五位奇数代表男，偶数代表女），15位身份证不含X
		//$结尾


		boolean matches = IDNumber.matches(regularExpression);
		//判断第18位校验值
		if (matches) {

			if (IDNumber.length() == 18) {
				try {
					char[] charArray = IDNumber.toCharArray();
					//前十七位加权因子
					int[] idCardWi = {7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2};
					//这是除以11后，可能产生的11位余数对应的验证码
					String[] idCardY = {"1", "0", "X", "9", "8", "7", "6", "5", "4", "3", "2"};
					int sum = 0;
					for (int i = 0; i < idCardWi.length; i++) {
						int current = Integer.parseInt(String.valueOf(charArray[i]));
						int count = current * idCardWi[i];
						sum += count;
					}
					char idCardLast = charArray[17];
					int idCardMod = sum % 11;
					if (idCardY[idCardMod].toUpperCase().equals(String.valueOf(idCardLast).toUpperCase())) {
						return true;
					} else {
						System.out.println("身份证最后一位:" + String.valueOf(idCardLast).toUpperCase() +
								"错误,正确的应该是:" + idCardY[idCardMod].toUpperCase());
						return false;
					}

				} catch (Exception e) {
					e.printStackTrace();
					System.out.println("异常:" + IDNumber);
					return false;
				}
			}

		}
		return matches;
	}

//	public static void main(String[] args) {
//		RegularPattern rp = new RegularPattern();
////		String tel="18295712928,15275553081,15375553081,15475553081,15575553081,15675553081,15775553081,15875553081,15975553081,15075553081";
////		Map<String,Object> map=rp.isMobileNOList(tel);
////		int bugCount=Integer.parseInt(map.get("bugCount").toString());
////		if(bugCount>0){
////			System.out.println(map.get("bugStr"));
////		}else{
////			System.out.println("无异常");
////		}
////		isMobileNO("13701306775,15311553878,17611265864,18611166556,15156765760,15056012862,13866980962,18895315865,13826526893,18388229676,18811390181,16601185575,18518986038,15210289658,18519711461,13141382052,18104374972,18832975635,18942309092,18384237126,15169322207,18302837324,18108292529,18190874609,15982325278,13739495515,17780687858,13551192356,15982321194,15982209300,18328328060,18602872515,18202823713,18384219626,15841030191,13795112669,15942426518,17731627807,15618086631,15566518456,18903167271,15761618511,15872209004,18645747878,18947410195,18780128717,13580310393,13682287983,15070177155,18310928603,18370990427,18770428979,13828450802,18818867713,15818851908,13202000439,15920583743,13560946621,18766470642,13560389149,13580411107,13609790995,13760752654,18928823378,13682282628,13265070075,13711726396,13632245469,13595052534,18890014530,13693109380,13520971169,13641139087,15875335103,13632770338,18513490130,15877599334,13819483200,15013486665,15005668122,15212413566,13965049873,15256580673,13647345642,13637341875,13576080994,15572588229,17674785769,15111132410,13517479731,13548581191,15580944453,15220600155,13723751641,15621141521,17667508166,15966850005,17854123855,15866760713,17605313664,18766110761,15153135671,18353196535,18366101966,13235318223,15662692576,13884989896,15098866577,18615672582,18766128881,18396851575,15806616785,15275735033,15879541996,18579189341,13033238854,13032996607,15161645150,18559269163,13510688098,15021120491,15880781541,13020082800,13797051799,15872194112,13184579698,15870806518,17307920990,18907920959,18788586550,13962689863,13731616746,15318836015,15263508166,13714558416,13684922126,18806661926,15914140032,18959074650,17721980860,13982713006,17612875408,18715702801,15908265612,13530349363,18170288877,18870072476,13697096732,15270883118,13707093380,15181777128,13382777795,13915921336,15855366045,18936769935,13770948947,18622961899,18322697136,18990081751,18754163231,17704625096,18765971979,15966866505,15620607110,13850067583,15259274290,13850085675,18359660911,13075813327,15980804348,15859256316,15960382837,15980991320,15280211721,13507293575,18396822800,18769707317,15552851355,13188882015,15628923209,18363071600,15554722253,15650310071,18653109020,18092873669,13259456385,13916959489,15901649845,13020208279,18817871586,13585559395,15900592145,18651324258,15801860025,17321251018,18169393231,15814607701,13926572900,18565624293,15919765943,15818770287,15112309673,15994728963,13713844470,18929395855,13418668470,15815526477,18598271965,13531232683,13714506477,13530299880,18829042471,13632918722,15011621850,13510741502,13632526330,13537537370,15014054504,15040310037,13750084397,15202043143,13682245703,13352910813,13590219889,15817445584,13723465696,13822583389,13699753423,13590360294,13530769801,18283020031,13656273026,13882179134,13982503396,18215638610,15962141552,15850195070,18861382027,15196926516,13330631688,18769880103,18565890803,13502171055,13821402850,15222340817,15122817001,18222219407,13622045895,13602105544,15822646029,13821343576,18322528174,18714419977,13002224619,15922165687,15510806481,13920710173,18202646418,13212278933,18630908013,15222141477,18622662795,13920168328,13512925168,18532110068,15190225668,15961816571,13955330995,13971592489,18602742010,18907149546,13607178307,15171472355,18971196978,13545050557,18674089098,15927132282,17612774094,15827399697,18560730856,13953144430,15205475136,13590351520,18579119542,13686408382,18344483620,17683928929,18201755070,18503003649,13661290480,15010582123,18046888788,18174668518,15123452620,13538226559,13907306755,15111211932,13507421646,13786115753,13974979842,18711332772,15970631717,17614500293,13521805360,15764506807,15561899395,17342950185,18664905921,13714022649,15928826320,17791835045,18292079763,13621273224,18822878369,15800900430,13425109790,15211102841,13754823533,13826578641,13534178885,18988776322,13630228819,15712164306,18825103911,13814253553,18255422763,15610170115,18663735023,18366617895,13064013516,18918667260,13636554604,15013641016,13299998831,13820522670,18702781808,13607199864,13329726482,13886008184,18842694268,18779183586,15998875437,17707489820,13265956298,15976231844,13823177789,15217288848,18511634822,15116200097,13723722627,15603979813,18375939872,13368084529,13160681609,13924707471,17765028458,18859661701,17611233015,15810115894");
//		String [] str="13701306775,15311553878,17611265864,18611166556,15156765760,15056012862,13866980962,18895315865,13826526893,18388229676,18811390181,16601185575,18518986038,15210289658,18519711461,13141382052,18104374972,18832975635,18942309092,18384237126,15169322207,18302837324,18108292529,18190874609,15982325278,13739495515,17780687858,13551192356,15982321194,15982209300,18328328060,18602872515,18202823713,18384219626,15841030191,13795112669,15942426518,17731627807,15618086631,15566518456,18903167271,15761618511,15872209004,18645747878,18947410195,18780128717,13580310393,13682287983,15070177155,18310928603,18370990427,18770428979,13828450802,18818867713,15818851908,13202000439,15920583743,13560946621,18766470642,13560389149,13580411107,13609790995,13760752654,18928823378,13682282628,13265070075,13711726396,13632245469,13595052534,18890014530,13693109380,13520971169,13641139087,15875335103,13632770338,18513490130,15877599334,13819483200,15013486665,15005668122,15212413566,13965049873,15256580673,13647345642,13637341875,13576080994,15572588229,17674785769,15111132410,13517479731,13548581191,15580944453,15220600155,13723751641,15621141521,17667508166,15966850005,17854123855,15866760713,17605313664,18766110761,15153135671,18353196535,18366101966,13235318223,15662692576,13884989896,15098866577,18615672582,18766128881,18396851575,15806616785,15275735033,15879541996,18579189341,13033238854,13032996607,15161645150,18559269163,13510688098,15021120491,15880781541,13020082800,13797051799,15872194112,13184579698,15870806518,17307920990,18907920959,18788586550,13962689863,13731616746,15318836015,15263508166,13714558416,13684922126,18806661926,15914140032,18959074650,17721980860,13982713006,17612875408,18715702801,15908265612,13530349363,18170288877,18870072476,13697096732,15270883118,13707093380,15181777128,13382777795,13915921336,15855366045,18936769935,13770948947,18622961899,18322697136,18990081751,18754163231,17704625096,18765971979,15966866505,15620607110,13850067583,15259274290,13850085675,18359660911,13075813327,15980804348,15859256316,15960382837,15980991320,15280211721,13507293575,18396822800,18769707317,15552851355,13188882015,15628923209,18363071600,15554722253,15650310071,18653109020,18092873669,13259456385,13916959489,15901649845,13020208279,18817871586,13585559395,15900592145,18651324258,15801860025,17321251018,18169393231,15814607701,13926572900,18565624293,15919765943,15818770287,15112309673,15994728963,13713844470,18929395855,13418668470,15815526477,18598271965,13531232683,13714506477,13530299880,18829042471,13632918722,15011621850,13510741502,13632526330,13537537370,15014054504,15040310037,13750084397,15202043143,13682245703,13352910813,13590219889,15817445584,13723465696,13822583389,13699753423,13590360294,13530769801,18283020031,13656273026,13882179134,13982503396,18215638610,15962141552,15850195070,18861382027,15196926516,13330631688,18769880103,18565890803,13502171055,13821402850,15222340817,15122817001,18222219407,13622045895,13602105544,15822646029,13821343576,18322528174,18714419977,13002224619,15922165687,15510806481,13920710173,18202646418,13212278933,18630908013,15222141477,18622662795,13920168328,13512925168,18532110068,15190225668,15961816571,13955330995,13971592489,18602742010,18907149546,13607178307,15171472355,18971196978,13545050557,18674089098,15927132282,17612774094,15827399697,18560730856,13953144430,15205475136,13590351520,18579119542,13686408382,18344483620,17683928929,18201755070,18503003649,13661290480,15010582123,18046888788,18174668518,15123452620,13538226559,13907306755,15111211932,13507421646,13786115753,13974979842,18711332772,15970631717,17614500293,13521805360,15764506807,15561899395,17342950185,18664905921,13714022649,15928826320,17791835045,18292079763,13621273224,18822878369,15800900430,13425109790,15211102841,13754823533,13826578641,13534178885,18988776322,13630228819,15712164306,18825103911,13814253553,18255422763,15610170115,18663735023,18366617895,13064013516,18918667260,13636554604,15013641016,13299998831,13820522670,18702781808,13607199864,13329726482,13886008184,18842694268,18779183586,15998875437,17707489820,13265956298,15976231844,13823177789,15217288848,18511634822,15116200097,13723722627,15603979813,18375939872,13368084529,13160681609,13924707471,17765028458,18859661701,17611233015,15810115894".split(",");
//		for(int i=0;i<str.length;i++){
//			System.out.println(i+""+isMobileNO(str[i]));
//		}
//		
////		String idcard="430227199503032621";
////		if(rp.isIDNumber(idcard)){
////			System.out.println("身份证有效");
////		}else{
////			System.out.println("身份证无效");
////		}
////		String email="13732960942@163.com";
////		boolean bool=rp.checkEmail(email);
////		if(bool){
////			System.out.println("邮箱无异常");
////		}else{
////			System.out.println("邮箱有异常，异常邮箱为："+email);
////		}
//	}
	
	public static void main(String[] args) {
		//至少6个字符，至少1个大写字母，1个小写字母和1个数字
		String pwdPattern="^(?=.*[a-z])(?=.*[A-Z])(?=.*\\d)[\\s\\S]{6,12}$";
		String pwd="Ch12345678";
		Pattern p = Pattern.compile(pwdPattern);
		Matcher m=p.matcher(pwd);
		if(m.matches()){
			System.out.println("密码格式满足");
		}else{
			System.out.println("密码格式不满足");
		}
	}

}
