package com.palmnest.application.core.util;

import com.google.gson.Gson;
import com.palmnest.application.dto.output.enterprise.VirOutDTO;
import org.junit.Test;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * 排序算法
 *
 * @author zhaoheng
 * @creation date 2018-09-12
 */
public class Sort {

	/**
	 * 字母大小写标识 capital:大写
	 */
	private static final String Letter_flag_capital = "capital";

	/**
	 * 企业内购 企业列表排序的方法
	 *
	 * @param list 需要排序的List集合
	 * @return
	 */
	public Map<String, List<VirOutDTO>> sortCompany(List<VirOutDTO> list) {
		Map<String, List<VirOutDTO>> map = new HashMap<String, List<VirOutDTO>>();// 封装的map结果集
		List<VirOutDTO> arraylist = new ArrayList<VirOutDTO>();// 正常企业名的集合
		List<VirOutDTO> specialList = new ArrayList<VirOutDTO>();// 特殊企业名的集合
		String[] alphatableb =
				{
						"A", "B", "C", "D", "E", "F", "G", "H", "I",
						"J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z"
				};

		// 正常企业名 a-z,A-Z
		for (String a : alphatableb) {
			for (int i = 0; i < list.size(); i++) {//为了排序都返回大写字母
				if (a.equals(String2AlphaFirst(list.get(i).getVirName().toString(), Letter_flag_capital))) {
					arraylist.add(list.get(i));
				}
			}
			map.put(a, arraylist);
			arraylist = new ArrayList<VirOutDTO>();
		}

		for (int i = 0; i < list.size(); i++) {
			if ("0".equals(String2AlphaFirst(list.get(i).getVirName().toString(), Letter_flag_capital))) {
				// 特殊企业名的集合 0-9;!@#$....等特殊字符开头的名称
				specialList.add(list.get(i));
			}
		}
		map.put("#", specialList);
		return map;
	}

	/**
	 * 排序的方法
	 *
	 * @param list 需要排序的List集合
	 * @return
	 */
	public Map<String, List<String>> sort(List<String> list) {
		Map<String, List<String>> map = new HashMap<String, List<String>>();// 封装的map结果集
		List<String> arraylist = new ArrayList<String>();// 正常名成的集合
		List<String> specialList = new ArrayList<String>();// 特殊名称的集合 例如特殊字符
		String[] alphatableb =
				{
						"A", "B", "C", "D", "E", "F", "G", "H", "I",
						"J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z"
				};
		for (String a : alphatableb) {
			for (int i = 0; i < list.size(); i++) {//为了排序都返回大写字母
				if (a.equals(String2AlphaFirst(list.get(i).toString(), Letter_flag_capital))) {
					arraylist.add(list.get(i).toString());
				}
			}
			map.put(a, arraylist);
			arraylist = new ArrayList<String>();
		}

		for (int i = 0; i < list.size(); i++) {
			if ("0".equals(String2AlphaFirst(list.get(i).toString(), Letter_flag_capital))) {
				// 特殊名称的集合 0-9;!@#$....等特殊字符开头的名称
				specialList.add(list.get(i).toString());
			}
		}
		map.put("#", specialList);

		return map;
	}

	//字母Z使用了两个标签，这里有２７个值
	//i, u, v都不做声母, 跟随前面的字母
	private char[] chartable =
			{
					'啊', '芭', '擦', '搭', '蛾', '发', '噶', '哈', '哈',
					'击', '喀', '垃', '妈', '拿', '哦', '啪', '期', '然',
					'撒', '塌', '塌', '塌', '挖', '昔', '压', '匝', '座'
			};

	// 大写字母匹配数组
	private char[] alphatableb =
			{
					'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I',
					'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z'
			};

	// 小写字母匹配数组
	private char[] alphatables =
			{
					'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i',
					'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z'
			};

	private int[] table = new int[27];  //初始化

	{
		for (int i = 0; i < 27; ++i) {
			table[i] = gbValue(chartable[i]);
		}
	}

	/**
	 * 主函数,输入字符,得到他的声母,英文字母返回对应的大小写字母,英文字母返回对应的大小写字母
	 *
	 * @param ch   字符
	 * @param type 大小写类型标识
	 * @return
	 */
	public char Char2Alpha(char ch, String type) {
		if (ch >= 'a' && ch <= 'z')
			return (char) (ch - 'a' + 'A');//为了按字母排序先返回大写字母

		if (ch >= 'A' && ch <= 'Z')
			return ch;
		int gb = gbValue(ch);
		if (gb < table[0])
			return '0';

		int i;
		for (i = 0; i < 26; ++i) {
			if (match(i, gb))
				break;
		}

		if (i >= 26) {
			return '0';
		} else {
			if (Letter_flag_capital.equals(type)) {//大写
				return alphatableb[i];
			} else {//小写
				return alphatables[i];
			}
		}
	}

	/**
	 * 根据一个包含汉字的字符串返回一个汉字拼音首字母的字符串
	 *
	 * @param SourceStr 目标字符串
	 * @param type      大小写类型
	 * @return
	 */
	public String String2Alpha(String SourceStr, String type) {
		String Result = "";
		int StrLength = SourceStr.length();
		int i;
		try {
			for (i = 0; i < StrLength; i++) {
				Result += Char2Alpha(SourceStr.charAt(i), type);
			}
		} catch (Exception e) {
			Result = "";
		}
		return Result;
	}

	/**
	 * 根据一个包含汉字的字符串返回第一个汉字拼音首字母的字符串
	 *
	 * @param SourceStr 目标字符串
	 * @param type      大小写类型
	 * @return
	 */
	public String String2AlphaFirst(String SourceStr, String type) {
		String Result = "";
		try {
			Result += Char2Alpha(SourceStr.charAt(0), type);
		} catch (Exception e) {
			Result = "";
		}
		return Result;
	}

	private boolean match(int i, int gb) {
		if (gb < table[i])
			return false;
		int j = i + 1;

		//字母Z使用了两个标签
		while (j < 26 && (table[j] == table[i]))
			++j;
		if (j == 26)
			return gb <= table[j];
		else
			return gb < table[j];
	}

	/**
	 * 取出汉字的编码
	 *
	 * @param ch
	 * @return
	 */
	private int gbValue(char ch) {
		String str = new String();
		str += ch;
		try {
			byte[] bytes = str.getBytes("GBK");
			if (bytes.length < 2)
				return 0;
			return (bytes[0] << 8 & 0xff00) + (bytes[1] &
					0xff);
		} catch (Exception e) {
			return 0;
		}
	}

	@Test
	public void test1() {
		List<VirOutDTO> list = new ArrayList<>();
		VirOutDTO d1 = new VirOutDTO();
		d1.setVirName("啊强企业");

		VirOutDTO d12 = new VirOutDTO();
		d12.setVirName("比特企业");

		VirOutDTO d2 = new VirOutDTO();
		d2.setVirName("沧桑企业");

		VirOutDTO d3 = new VirOutDTO();
		d3.setVirName("得体企业");

		VirOutDTO d4 = new VirOutDTO();
		d4.setVirName("恩啊好企业");

		VirOutDTO d5 = new VirOutDTO();
		d5.setVirName("妇代会企业");

		VirOutDTO d6 = new VirOutDTO();
		d6.setVirName("咕噜企业");

		VirOutDTO d7 = new VirOutDTO();
		d7.setVirName("黄是企业");

		VirOutDTO d8 = new VirOutDTO();
		d8.setVirName("才到家扩散强企业");

		VirOutDTO d9 = new VirOutDTO();
		d9.setVirName("0-0企业");

		VirOutDTO d10 = new VirOutDTO();
		d10.setVirName("  .强企业");

		VirOutDTO d11 = new VirOutDTO();
		d11.setVirName("!@#企业");

		list.add(d1);
		list.add(d2);
		list.add(d3);
		list.add(d4);
		list.add(d5);
		list.add(d6);
		list.add(d7);
		list.add(d8);
		list.add(d9);
		list.add(d10);
		list.add(d11);
		list.add(d12);

		Map<String, List<VirOutDTO>> stringListMap = sortCompany(list);
		Gson gson = new Gson();
		System.out.println(gson.toJson(stringListMap));
	}
}

