package com.palmnest.webapp.quartz;

import com.palmnest.service.repair.RepairBillService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * 维修单推送定时任务
 * <p>
 * 3.在某个在挂单状态的维修单，超过预计重新开始时间的5分钟后，对应的工程人员未及时处理，拥有“管理维修单”功能的人员，收到此通知。若此维修单一直未变状态（“挂单中”），则每5分钟发一次提醒。
 *
 * @author wangxz
 * <p>
 * 2018年5月15日
 */
@Slf4j
@Component("repairManagePendBillPushTimerTask")
public class RepairManagePendBillPushTimerTask {


	@Autowired
	private RepairBillService repairBillService;

	public void run() {
		//System.out.println("3.在某个在挂单状态的维修单，超过预计重新开始时间的5分钟后，对应的工程人员未及时处理，拥有“管理维修单”功能的人员，收到此通知。若此维修单一直未变状态（“挂单中”），则每5分钟发一次提醒。");
		log.info("quartz into RepairManagePendBillPushTimerTask start ...");

		//查找挂单时间大于当前时间的

		try {
			repairBillService.managePendBillPush();
		} catch (Exception e) {
			e.printStackTrace();
			log.error(e.getMessage());
		}

		log.info("quartz into RepairManagePendBillPushTimerTask end ...");


	}
}
